<?php
	 /*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once "csnippet.phps";

	/**	Calendar class.
	 *	<xmp>
	 |	<?php
	 |	$calendar = new CCalendar();
	 |	?>
	 |	...
	 |	<table>
	 |	<?php
	 |	$calendar->render(getdate(), array(
	 |		array (
	 |			"year"	=> 2001, "mon" => 5, "mday" => 1,
	 |			"href"	=> "http://events.com/may/party.html",
	 |			"text"	=> "Welcome to Funny Party!"
	 |		),
	 |		...
	 |	));
	 |	?>
	 |	</table>
	 *	</xmp>
	 *	The presentation is controlled by CSS:
	 *	<pre><ul>
	 *	<li> .calmonth	- month class
	 *	<li> .calweek	- weekday class
	 *	<li> .calempty	- 'empty' day class
	 *	<li> .calday	- ordinary day class
	 *	<li> .caltoday	- today's class
	 *	<li> .calhref	- link a-tag class
	 *	<li> .caltext	- link text class
	 *	</ul></pre>
	 *	@package	PHP/Toolbox
	 */
	class CCalendar extends CSnippet {
		var $empcell = "&nbsp;";
		var $weekdays = array("Mo","Tu","Wd","Th","Fr","St","Su");
		var $monthdays = array(0,31,28,31,30,31,30,31,31,30,31,30,31);

		function CCalendar ($weekdays=array(), $empcell="") {
			if ($empcell) $this->empcell = $empcell;
			if ($weekdays) $this->weekdays = $weekdays;
		}

		function render ($thedate=0, $linkdays=array()) {
			$this->render_month ($thedate, $linkdays);
		}

		/**	Renders specified month.
		 * 	@param	int		- the date (e.g. getdate($thedate));
		 * 					  the current date (getdate()) is used if it's not specified (0).
		 * 	@param	array	- array of links to specified dates; each link is:
		 *  <pre>
		 *	array (
		 * 		"year"	=> 2001,	// event year
		 * 		"mon"	=> 5,		// event month
		 * 		"mday"	=> 1,		// event day in the month
		 * 		"href"	=> "http://events.com/may/party.html",	// the link
		 * 		"text"	=> "Welcome to Funny Party!"			// the link text
		 * 	);
		 *  </pre>
		 * 	The link text can be omitted, if not then &lt;DATE&gt; template tag will be
		 * 	substituted with the link's date.
		 * 	It's also possible to assign several links to one date -- they all will be listed.
		 */
		function render_month ($thedate=0, $linkdays=array()) {
			$today = getdate(time());

			$thedate = $thedate ? getdate($thedate) : getdate();
			$theweekday		= $thedate["wday"];
			$themonth  		= $thedate["mon"];
			$themonthday	= $thedate["mday"];
			$theyear   		= $thedate["year"];

			$theyear = $theyear % 100;
			$theyear = (($theyear < 50) ? (2000 + $theyear) : (1900 + $theyear));
			if ((($theyear % 4 == 0) && !($theyear % 100 == 0))||($theyear % 400 == 0))
    			$this->monthdays[2]++;

			$iscurrent	=	$themonth		== $today["mon"]
						&&	$themonthday	== $today["mday"]
						&&	$theyear   		== $today["year"];

			$startspaces = $themonthday;
			$startspaces = $theweekday - $themonthday % 7;
			if ($startspaces < 0) $startspaces += 7;

			$this->write("<tr><td class=\"calmonth\" colspan=7>"
    			."$thedate[month] $theyear</td></tr>");
			$this->write("<tr>");
			for ($i=0; $i < 7; $i++) {
				$this->write("<td class=\"calweek\">".$this->weekdays[$i]."</td>");
			}
			$this->write("</tr><tr>");
			for ($s=0; $s < $startspaces; $s++) {
    			$this->write("<td class=\"calempty\">$this->empcell</td>");
			}

			$count=1;
			while ($count <= $this->monthdays[$themonth]) {
    			for ($b = $startspaces; $b < 7; $b++) {
        			$this->write("<td class=");
					if ($count <= $this->monthdays[$themonth]) {
						$html = $count;
						if ($iscurrent && $count == $today["mday"]) {
							$this->write("\"caltoday\"");
						}
						else {
							$this->write("\"calday\"");
						}
					}
					else {
						$html = $this->empcell;
						$this->write("\"calempty\"");
            		}
					$this->write(">");
        			for ($c = 0; $c < count($linkdays); $c++) {
            			if (!empty($linkdays[$c])) {
                			if (($linkdays[$c]["year"] == $theyear) &&
								($linkdays[$c]["mon"]  == $themonth) &&
								($linkdays[$c]["mday"] == $count)) {
								$href = @$linkdays[$c]["href"];
								$text = @$linkdays[$c]["text"];
								if (empty($text)) {
									$html = "<a class=calhref href=$href>$html</a>";
							 		break;
								}
								else {
                    				$html .= "<br><a class=calhref href=$href><font class=caltext>$text</font></a>";
								}
                			}
            			}
        			}
					$html = str_replace("<DATE>", "$theyear-$themonth-$count", $html);
					$this->write($html);
        			$this->write("</td>");
        			$count++;
    			}
    			$this->write("</tr>");
    			$startspaces=0;
			}
		}
	}
?>