<?php
	/*'*********************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once	"cmarkup.phps";

	/**	Basic form control class.
	 * 	Use it to render an input tag somehow like
	 * 	<xmp>
	 | 	new CControl(array(
	 | 		"type"	=> "text", "img", etc.
	 | 		...
	 | 	))
	 * 	</xmp>
	 * 	@package	PHP/Toolbox::Form
	 */
	class  CControl extends CMarkup {
		var	$type = "";
		/**	Specifies that it's not a data control.
		 * 	I.e., there is no direct correspondence to
		 * 	some database field.
		 */
		var $nodata = FALSE;

		/**	Constructor.
		 */
		function CControl ($attrs = array()) {
			$this->CMarkup($attrs);
		}

		/**	Main rendering method.
		 * 	It never has to be used directly, use it via the parent form method
		 *	<pre>$form->renderControl($form->controls["name"]);</pre>
		 * 	instead.
		 */
		function render () {
			$value = $this->getValue();
			$this->write("<input name='$this->name' type='$this->type' value='$value' $this->attrs>");
		}

		/**	Readonly rendering method.
		 * 	It's called by parent form when in Readonly mode.
		 * 	It produces a plain text and a hidden field.
		 */
		function renderReadonly () {
			$value = $this->getValue();
			if ($this->type !== "hidden") {
				$this->write($value);
			}
			$this->write("<input name='$this->name' type='hidden' value='$value'>");
		}

		/**	Returns control's value.
		 * 	Use this also to override the data handling,
		 * 	e.g. to process special data formats.
		 *	Note, stripslashes allow us to ignore magic_quotes_gpc setting.
		 * 	@return	mixed	- value.
		 */
		function getValue () {
			$value = $this->getGlobal($this->name, @$this->default);
			if (!is_array($value)) {
				return stripslashes($value);
			}
			$newval = array();
			foreach ($value as $v) {
				$newval[] = stripslashes($v);
			}
			return $newval;
		}

		/**	Control's value to output from the form.
		 *	Use this to override the data handling,
		 *	e.g. to process special data formats.
		 *	@return	mixed	- out value.
		 */
		function outValue () {
			//	note, addslashes allows us to ignore magic_quotes_gpc setting
			$value = $this->getValue();
			if (!is_array($value)) {
				return addslashes($value);
			}
			$newval = array();
			foreach ($value as $v) {
				$newval[] = addslashes($v);
			}
			return $newval;
		}

		/**	Control's value to input to form.
		 *	Use this to override the data handling,
		 *	e.g. to process special data formats.
		 * 	<xmp>
		 |	$rset = do_query ($sql);
		 |	while (fetch_record($rset, $rec)) {
		 |		foreach ($rec as $fname => $value) {
		 |			global $$fname;
		 |			$$fname = inValue($value);
		 |		}
		 |	}
		 * 	...
		 |	function inValue ($value) {
		 |		//	handle the set
		 |		return split(",", $value);
		 |	}
		 * 	</xmp>
		 *	@param	mixed	- value.
		 *	@return	mixed	- in value ($value by default).
		 */
		function inValue ($value) {
			return $value;
		}

		/**	Verifies the control's data.
		 * 	Use this to override the data verification,
		 *	e.g. to process special data formats.
		 * 	Note that error message is eval'ed, so, you
		 * 	can use "\$value" to insert error data.
		 *	@return	mixed	- control validation result.
		 */
		function validateValue () {
			$value = trim($this->getValue());
			if ($value == "")
				$msg = @$this->ValidateAsNotEmpty ? $this->ValidateAsNotEmpty : "";
			elseif (@$this->ValidateAsInteger	&& !ereg('^[+-]?[0-9]*$', $value))
				$msg = $this->ValidateAsInteger;
			elseif (@$this->ValidateAsFloat		&& !ereg('^[+-]?[0-9]+(\\.[0-9]*)?([Ee][+-]?[0-9]+)?$', $value))
				$msg = $this->ValidateAsInteger;
			elseif (@$this->ValidateAsEmail		&& !ereg('^[a-z0-9_\\-\\.]+@[a-z0-9_\\-\\.]+$', $value))
				$msg = $this->ValidateAsEmail;
			elseif (@$this->ValidateAsRegex		&& !ereg($this->ValidationRegex, $value))
				$msg = $this->ValidateAsRegex;
			@eval ("\$msg = \"$msg\";");
			return $msg;
		}
	}
?>