<?php
	/*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	if (!defined("MAX_FIELD_LENGTH")):
	define("MAX_FIELD_LENGTH", 250);
	endif;//MAX_FIELD_LENGTH

	require_once "cform.phps";

	/**	Basic database-driven form class.
	 * 	The main idea is to use SQL-query meta-information
	 * 	to automatically generate appropriate form fields.
	 * 	On construction the controls may be put into control list in order
	 * 	<ul>
	 * 	<li>	to provide the user interface to named field that exists in the query;
	 * 	<li>	to <i>ignore</i> such a field;
	 * 	<li>	to provide the user interface to virtual field that isn't in the query but should be;
	 * 	<li>	to provide something like buttons, e.g. 'reset'.
	 * 	</ul>
	 * 	@see	addControls()
	 * 	@see	get_infield()
	 * 	@package	PHP/Toolbox::Form
	 */
	class CDBForm extends CForm {

		/**	Overrides parent method to provide the special functionality.
		 * 	In partucular, controls with 'dontadd' field set are skipped.
		 */
		function addControls ($controls) {
			foreach ($controls as $name => $control) {
				if (@$control->dontadd) {
					continue;
				}
				$this->addControl($name, $control);
			}
		}

		/**	Take control from input list
		 * 	and check whether the field is to be ignored
		 * 	(fields with 'ignore' field set are skipped).
		 * 	Otherwise, create the new CControl.
		 * 	@param	array	- the list of form controls.
		 * 	@param	string	- field name.
		 * 	@return	object CControl
		 */
		function get_infield (&$FIELD_DESCR, $fname) {
			if ($infield = @$FIELD_DESCR[$fname]) {
				$FIELD_DESCR[$fname] = 0;
				if (@$infield->ignore)
					return NULL;
			}
			else {
				$infield = new CControl();
			}
			return $infield;
		}

		/**	Sets up the control (infield) by providing it with:
		 * 	<ul>
		 * 	<li>	the field name;
		 * 	<li>	the display label;
		 * 	<li>	the table the field is originated from;
		 * 	<li>	validation data if the field is required;
		 * 	<li>	db lookup data if the field is a looking up one.
		 * 	</ul>
		 * 	if they all aren't already set.
		 * 	@param	object CControl	- inout field control.
		 * 	@param	string	- field name.
		 * 	@param	string	- guess!
		 * 	@param	bool	- guess!
		 */
		function setup_infield (&$infield, $fname, $table, $required) {
			set_if_empty($infield, "name",	strtr($fname, ". ", "__"));
			set_if_empty($infield, "label",	strtr($fname, "_", " "));
			set_if_empty($infield, "table",	$table);
			if ($required) {
				$infield->required = TRUE;
				set_if_empty($infield, "ValidateAsNotEmpty", REQUIRED);
			}
			if (@$infield->lookup) {
				$lookup = $infield->lookup;
				if (isset($this->controls[$lookup]))
					set_lookup_options($infield, $this->fields[$lookup]->options);
				else
					set_lookup_options($infield);
			}
		}

		/**	Constructs the SQL 'set' clause list with the fields that
		 * 	belong to specified table, e.g.:
		 * 	<xmp>
		 |	do_query("insert into table_1 set ".setList("table_1");
		 * 	</xmp>
		 * 	@access	public
		 */
		function setList ($table = "") {
			$list = $comma = "";
			foreach ($this->controls as $fname => $infield) {
				//PHPX_DEBUG($infield);
				if (@$infield->nodata)
					continue;
				if (@$infield->table !== $table)
					continue;
				$value = $this->outValue($infield);
				if (!$this->process_specially($infield, $value))
					continue;
				$list .= "$comma$fname=$value";
				$comma = ",";
			}
			return $list;
		}
	}

	/**	Sets up the lookup data to specified infield.
	 * 	If the options isn't specified excplicitely, executes the lookup query
	 * 	and sets the lookup data as key=>value pairs.<br>
	 * 	Note the lookup query format:
	 * 	<pre>
	 | 	new CSelect(array( # for example
	 |		"lookup" => "looked_key_field as <b>op_label</b>, looked_value_field as <b>op_value</b> from looked_table"
	 |		...
	 |	));
	 * 	</pre>
	 * 	Hope, it's clear enough.
	 * 	@param	object CControl	- inout field control.
	 * 	@param	array	- to specify the options excplicitely.
	 */
	function set_lookup_options (&$infield, $options=0) {
		if (is_integer($options)) {
			$query = do_query("select $infield->lookup");
			$options = array();
			if (!$infield->multiple) $options[0] = "";
			while (fetch_object($query, $rec)) {
				$options[$rec->op_label] = $rec->op_value;
			}
			free_result($query);
		}
		//PHPX_DEBUG($options);
		//PHPX_DEBUG($infield);
		set_if_empty($infield, "options", $options);
		set_if_empty($infield, "default", "0");
	}

?>