<?php
	/*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once "cdbform.phps";

	/**	Database query-driven form class.
	 * 	The main idea is to use SQL-query meta-information
	 * 	to automatically generate appropriate form fields.
	 * 	<xmp>
	 |	$form = new CDBQueryForm(FORM_HTML_HANDLER,
	 |		"select a, b, c, format_date("%r", date) as full_date from table_1", "where",
	 |		$PHP_SELF, "get"
	 |	);
	 |	$form->build(array(
	 |		"field_1" => new CControl(array(
	 |			...
	 |		)),
	 |		...
	 |	));
	 |	...
	 |	$form->render();
	 * 	</xmp>
	 * 	@package	PHP/Toolbox::Form
	 */
	class CDBQueryForm extends CDBForm {
		var $sql, $where;

		/**	Constructs the form from the query.
		 * 	@param	string	- form_html_handler (see CForm).
		 * 	@param	string	- sql-statement to execute. It can be as simple or as complex you like.
		 * 	@param	string	- fetching use its own where clause that is simply appended to sql-string above.
		 * 					So, if 'where' clause is already specified in sql-stmt it's possible to specify
		 * 					fetching as additional condition -- for instance as 'and'-operator.
		 * 	@param	string
		 * 	@param	string
		 * 	@param	array	- additional fields just like in CForm and CSnippet.
		 */
		function CDBQueryForm ($handler, $sql="", $where="where", $action="", $method="get", $attrs="") {
			$this->CForm($handler, array(), $action, $method, $attrs);
			$this->sql = $sql;
			$this->where = $where;
		}

		/**	Builds the control list
		 * 	using the form sql-query meta-info.
		 * 	@param	array
		 * 	@access	public
		 */
		function build ($FIELD_DESCR=array()) {
			$fields = array();
			$rset = do_query("$this->sql limit 0,0");
			for ($i=0; fetch_field($rset, $field, $i); $i++) {
				$fields[$field->name] = $field;
			}
			free_result($rset);

			foreach ($fields as $fname => $field) {
				if (!$infield = $this->get_infield($FIELD_DESCR, $fname))
					continue;
				$this->setup_infield($infield, $fname, $field->table, $field->not_null);
				switch ($field->type) {
					case "int" :
						set_if_empty($infield, "ValidateAsInteger", NOT_AN_INTEGER);
						break;
				}
				if ($field->max_length) {
					set_if_empty($infield, "maxlength", max($field->max_length,	MAX_FIELD_LENGTH));
				}
 				$this->addControl($fname, $infield);
			}
			$this->addControls($FIELD_DESCR);
		}

		/**	Fetches controls' values from the query.
		 * 	It performs some std processing, like converting csv-lists into arrays
		 * 	and uses control's inValue() to possibly customize the value processing as well.
		 * 	@param	string	- fetching use its own where clause that is simply appended to form sql-string:
		 * 					<code>"$this->sql $this->where $where"</code>
		 * 	@see	CDBQueryForm()
		 * 	@access	public
		 */
		function fetchValues ($where) {
			if (!$this->sql) return;
			$rset = do_query ("$this->sql $this->where $where");
			if (fetch_record($rset, $rec)) {
				foreach ($rec as $fname => $value) {
					$control = @$this->controls[$fname];
					if (!$control) {
						continue;
					}
					if (@$control->multiple === TRUE) {
						$value = split(",", $value);
					}
					global $$fname;
					$$fname = $this->inValue($control, $value);
				}
			}
			free_result($rset);
		}

		/**	Provide custom output (to be setList'ed) value processing.
		 * 	By default:
		 * 	<ul>
		 * 	<li>	if value isn't set returns 'null';
		 * 	<li>	if field type is a 'password' returns password('value')
		 * 			<br>(it's not quite sequre and is subject to polish);
		 * 	<li>	if value is an array joins it with ','.
		 * 	</ul>
		 * 	@param	object CControl	- in.
		 * 	@param	mixed	- inout.
		 * 	@return	mixed	- value.
		 */
		function process_specially(&$infield, &$value) {
			if (!isset($value) || $value === "")  // AI 15.05.2001 19:36
				$value = "null";
			elseif ($infield->type === "password")
				$value = "password('$value')";
			elseif (is_array($value))
				$value = "'".join($infield->multiple ? "," : "','", $value)."'";
			else $value = "'$value'";
			return $value;
		}
	}
?>