<?php
	/*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once	"cdbqueryform.phps";

	/** Uses "show fields from table" statement
	 *  to generate	a form for given table.
	 * 	<xmp>
	 |	$form = new CDBQueryForm(FORM_HTML_HANDLER,
	 |		"table_1",
	 |		$PHP_SELF, "get"
	 |	);
	 |	$form->build(array(
	 |		"field_1" => new CControl(array(
	 |			...
	 |		)),
	 |	...
	 |	));
	 |	...
	 |	$form->render();
	 * 	</xmp>
	 * 	@package	PHP/Toolbox::Form
	 */
	class CDBTabForm extends CDBQueryForm {
		var $table;
		/**	If the group items count exceeds this value
		 * 	then CSelect is used, otherwise -- CGroup.
		 * 	@see CSelect, CGroup
		 */
		var	$groupSizeThreshold = 4;
		/**	If summary group items length exceeds this value
		 * 	then CSelect is used, otherwise -- CGroup.
		 * 	@see CSelect, CGroup
		 */
		var	$groupLenThreshold = 40;

		/**	Constructs the form.
		 * 	The call format is similar to base class.
		 * 	@see CDBQueryForm
		 */
		function CDBTabForm ($handler, $table="", $action="", $method="get", $attrs="") {
			$this->table = $table;
			$this->CDBQueryForm ($handler, "", "where", $action, $method, $attrs);
		}

		function build ($FIELD_DESCR=array()) {
			//	add inputs according to table field descriptions //
			$fields = array();
			$rset = do_query("show fields from $this->table");
			while (fetch_object($rset, $field)) {
				$name = $fname = $field->Field;
				if ($field->Extra == "auto_increment")
					continue;
				if (!$infield = $this->get_infield($FIELD_DESCR, $fname))
					continue;
				$this->setup_infield($infield, $fname, $this->table, !$field->Null);

				$params = array();
				ereg("([[:alnum:]]+)(\\((.+)\\))* *([[:alnum:]]*)",	$field->Type, $params);
				/***  \\1=type, \\3=length/enum/set, \\4=attrib ***/
				//PHPX_DEBUG($params);

				if (empty($infield->type)) {
					switch ($params[1]) {
						case "set":
							$infield->multiple = TRUE;
						case "enum":
							$estlen = strlen($params[3]);
							$optlist = split(",", str_replace("'", "", $params[3]));
							$size = count($optlist);
							$options = array();
							if (@$infield->multiple === TRUE)
								$infield->attrs = "size='$size'";
							elseif (@$field->Null)
								$options["0"] = "";
							while (list($i,$temp)=each($optlist)) {
								$options[$temp] = $temp;
							}
							$infield->options = $options;
							set_if_empty($infield, "default", $optlist[0]);
							if ($size > $this->groupSizeThreshold)
								$infield = new CSelect($infield);
							else {
								if ($estlen > $this->groupLenThreshold) {
									$infield->suffix = "<br>";
								}
								$infield = new CGroup($infield);
							}
							//PHPX_DEBUG($infield);
							break;
						case "int" :
							set_if_empty($infield, "ValidateAsInteger", NOT_AN_INTEGER);
							break;
						case "timestamp" :
							set_if_empty($infield, "mode", "Readonly");
							break;
					}
				}
				if (intval($params[3])) {
					set_if_empty($infield, "maxlength", min($params[3],	MAX_FIELD_LENGTH));
				}
				if (isset($field->Default)) {
					set_if_empty($infield, "default", $field->Default);
				}
 				$this->addControl($fname, $infield);
			}
			free_result($rset);
			$this->addControls($FIELD_DESCR);
		}

		function fetchValues ($where) {
			$this->sql = "select * from $this->table";
			return CDBQueryForm::fetchValues($where);
		}

		function setList ($table="") {
			return CDBQueryForm::setList($this->table);
		}
	}
?>