<?php
	/*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once	"cdbtable.phps";

	/**	The template-based table snippet class
	 * 	that is db-driven just like CDBForm.
	 *  For example:
	 * 	<xmp>
	 |	$table = new CDBTemplateTable("T2",
	 |		//HEADER//
	 |		"<tr><td class=navbar colspan=...><NAVIGATOR></td></tr>"
	 |		"<tr class=header>",
	 |			."<td class=col1><SORT FIELD1></td>"
	 |			."<td class=col2><SORT FIELD2></td>"
	 |			."<*>"
	 |		."</tr>",
	 |		//ROW//
	 |		"<tr class=row<ROW>>",
	 |			."<td>           <FIELD1></td>"
	 |			."<td class=col2><FIELD2></td>"
	 |			."<*>"
	 |		."</tr>",
	 |		//FOOTER//
	 |		"<tr><td class=navbar colspan=...><NAVIGATOR></td></tr>"
	 |	);
	 * 	</xmp>
	 * 	It uses template substitution technique to generate a code:
	 *	<xmp>
	 *	Header/Footer:
	 *	-	<NAVIGATOR> - the whole splitbar.
	 *	-	<SORT /FIELD_NAME/>...</SORT> - sort by /FIELD_NAME/.
	 *	- 	<*> - the fields, not explicitely mentioned by SORT tags.
	 *	-	<INS>...</INS> - insert record text 'button'.
	 *
	 *	Data row:
	 *	-	<EDIT>...</EDIT> - edit record text 'button'.
	 *	-	<DEL>...</DEL> - delete record text 'button'.
	 *	-	<ROW> - 1 for odd and 2 for even rows.
	 *	-	<NUM> - row (NOT record!) number.
	 *	-	</FIELD_NAME/> - value of /FIELD_NAME/.
	 *	-	<*> - values of fields , not explicitely mentioned by previous tag.
	 *	-	<WHERE> - 'where' condition uniquely identifying the given record.
	 *	</xmp>
	 * 	@package	PHP/Toolbox::Table
	 */
	class CDBTemplateTable
	extends CDBTable
	{
		var	$HEADER, $ROW, $FOOTER;

		/**	Constructs the table.
		 * 	@param	string
		 * 	@param	string	- sql-statement.
		 * 	@param	string	- header template.
		 * 	@param	string	- data row template.
		 * 	@param	string	- footer template.
		 * 	@param	array
		 * 	</xmp>
		 */
		function CDBTemplateTable ($id, $QUERY, $HEADER, $ROW, $FOOTER="", $extra = array()) {
			$this->CDBTable ($id, $QUERY, $extra);
			if (!$extra["SPLITBAR"]) {
				$this->SPLITBAR = "<tr><td colspan=30 align=right class=navbar>$this->SPLITBAR</td></tr>";
			}
			$this->HEADER = $HEADER;
			$this->ROW = $ROW;
			$this->FOOTER = $FOOTER ? $FOOTER : $HEADER;
		}

		function render () {
			$SORT = $this->qualify("sort");
			$SDIR = $this->qualify("sdir");
			global	$$SORT, $$SDIR;

			$this->do_query_size();
			$this->do_query_page($rset, $$SORT, $$SDIR);
			$this->fetch_fields($rset);

			$outstr = str_replace("<NAVIGATOR>", ($this->PAGECOUNT > 1) ? $this->navstr : "", $this->subst_globals($this->HEADER));

			#//////	PRINT TABLE HEADER //////#

			reset($this->fields);
			while (list($col,$field) = each($this->fields)) {
				$fname = $field->name;
				$fd = @$this->FIELD_DESCR[$fname];
				$key = "<SORT ".strtoupper($fname).">";
				$replacement = "<a href='$PHP_SELF?".query_string(array(
					$SORT => $fname,
					$SDIR => $fname == $$SORT ? !$$SDIR : $fd->defsort
				))."'>";
				if (!ereg("<".strtoupper($fname).">", $this->ROW)) {
					$key = "<*>";
					$replacement = "<th> $replacement $fname </a></th>$key";
				}
				$outstr = str_replace($key,	$replacement, $outstr);
			}
			$outstr = str_replace("</SORT>", "</a>", $outstr);
			$outstr = str_replace("<INS>",
				"<a href='$this->FORMFILE?T=$this->id&".query_string(array("DO"=>"INS"))."'>",
				$outstr);
			$outstr = str_replace("</INS>", "</a>", $outstr);
			$outstr = str_replace("<*>", "", $outstr);
			$this->write("$outstr\n");

			#//////	PRINT ROWS //////#

			if ($this->TOTROWS) {
				for ($rn = 0; fetch_record($rset, $rec); $rn++) {
					$outstr = $this->subst_globals($this->ROW);
					$outstr = str_replace("<ROW>", ($rn & 1) + 1, $outstr);
					$outstr = str_replace("<NUM>", $this->PGOFFS + $rn, $outstr);
					$where = array(); reset($this->fields);
					// enumerate fields //
					while (list($col, $field) = each($this->fields)) {
						/*------------------------------*/
						$fname = $field->name;
						$value = $rec[$fname];
						$fd = @$this->FIELD_DESCR[$fname];
						if ($fd->primary || $field->primary_key) {
							$where[] = "$field->table.$fname='$value'";
						}
						$value = htmlspecialchars($value);
						/*------------------------------*/
						$this->process_specially ($fd, $value);
						$outstr = str_replace("<NULL>", $value != "" ? "" : "0", $outstr);
						$key = "<".strtoupper($fname).">";
						$replacement = $value != "" ? $value : "&nbsp;";
						if (!ereg($key, $outstr)) {
							$key = "<*>";
							$replacement = "<td> $value </td>$key";
						}
						$outstr = str_replace($key, $replacement, $outstr);
					}

					if ($where) {
						$qstr = $this->identify ($where);
						$outstr = str_replace("<WHERE>",  "$qstr", $outstr);
						$outstr = str_replace("<EDIT>", "<a href='$this->FORMFILE?T=$this->id&DO=EDIT&$qstr'>", $outstr);
						$outstr = str_replace("<DEL>",  "<a href='$this->FORMFILE?T=$this->id&DO=DEL&$qstr'>", $outstr);
						$outstr = ereg_replace("</(EDIT|DEL)>", "</a>", $outstr);
					}
					$outstr = str_replace("<*>", "", $outstr);
					$this->write("$outstr\n");
				}

				#//////	PRINT THE FOOTER AND FINISH //////#

				if ($this->PAGECOUNT > 1) {
					$this->write(str_replace("<NAVIGATOR>", $this->navstr, $this->subst_globals($this->FOOTER))."\n");
				}
			}
			else {
				$this->write("<tr><td colspan=100>".@EMPTY_QUERY."</td></tr>\n");
			}
			//	cleanup
			@free_result($rset);
		}/*render*/
	}/*CDBTemplateTable*/
?>