<?php
	/*'*************************************************\
		PHP4/X
		cform.phps
	 ****************************************************
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	//require_once "func.phps"; //DEBUG//
	require_once "csnippet.phps";

	/**	@const NO_INDEX - use it to specify no index to
	 *  form addControl() method.
	 * 	@see addControl()
	 */
	define	("NO_INDEX",	-1);

	#//////	FORMS /////////////////////////////////////////////////////////#

	/**	The main form class.
	 * 	The general CForm usage follows:
	 * 	<xmp>
	 |	$form = new CForm("Html_Handler_Suffix", array(
	 |		"field_1"	=> new CControl(array(
	 |			"label"	=> "Field 1",
	 |			...
	 |		)),
	 |		"field_2"	=> new CControl(array(
	 |			"label"	=> "Field 2",
	 |			...
	 |		)),
	 |		...
	 |	));
	 |	$form->addSubmitCancel("Submit_Label", "Cancel_Label");
	 |	$form->render();
	 * 	</xmp>
	 *  Html_Handler_Suffix specifies the class
	 *  CFormHandler{Html_Handler_Suffix} that's
	 *  responsible for the form rendering.
	 *  It's also possible to render the form manually:
	 *  <xmp>
	 |	<table>
	 |  	<tr>
	 |  		<td>Label</td>
	 |  		<td><?$form->renderControl($form->controls["name"]);?></td>
	 |  	</tr>
	 |	</table>
	 *  </xmp>
	 * 	@package	PHP/Toolbox::Form
	 */
	class CForm extends CSnippet {
		/**	Form mode.
		 * 	The additional mode supported by the form core is Readonly.
		 * 	@see setMode()
		 */
		var $mode = "";
		/**	Form action. */
		var $action;
		/**	Form method.
		 * 	GET, POST. */
		var $method;
		var $controls;

		#//////	CONSTRUCTION //////#

		/**	Constructor.
		 * 	@param string	$handler	- name of the form html handler.
		 * 	@param array	$controls	- array of controls.
		 * 	@param string	$action		- form action.
		 * 	@param string	$method		- form method.
		 * 	@param array	$attrs		- form extra attrs
		 * 	@see CSnippet::CSnippet()
		 */
		function CForm ($handler, $controls = array(), $action = "", $method='get', $attrs="") {
			global $PHP_SELF;
			$this->addControls($controls);
			$this->action = $action ? $action : $PHP_SELF;
			$this->method = $method;
			$handler = "CFormHandler$handler";
			$this->o = new $handler();
			$this->attrs = $attrs;
		}

		/**	Adds the list of controls.
		 * 	@access	public.
		 * 	@see addControl().
		 */
		function addControls ($controls) {
			foreach ($controls as $name => $control) {
				$this->addControl($name, $control);
			}
		}

		/**	Adds the control.
		 * 	First the control is checked on null.
		 * 	If the name is string then control is associated with this name.
		 * 	Otherwise, the name field of control is used.
		 * 	Otherwise, the control is added to the end of the list.
		 * 	@param string	- name of the control.
		 * 	@param object CControl  - the control.
		 */
		function addControl ($name, $control) {
			if (!$control) {
				return;
			}
			if (is_string($name)) {
				if (!isset($control->name)) {
					$control->name = $name;
				}
				$this->controls [$name] = $control;
			}
			elseif (isset($control->name)) {
				$this->controls [$control->name] = $control;
			}
			else {
				$this->controls [] = $control;
			}
		}

		#//////	SPECIAL CONSTRUCTION CASES //////#

		/**	Adds vars from the specified array.
		 * 	Uses HTTP_GET_VARS and type=hidden by default.
		 * 	@access	public
		 * 	@param string $HTTP_VARS	- array's *name*.
		 * 	@param string $type			- controls' type.
		 */
		function addVars ($HTTP_VARS = "HTTP_GET_VARS", $type = "hidden") {
			global $$HTTP_VARS;
			foreach ($$HTTP_VARS as $name => $value) {
				if (empty($this->controls[$name])) {
					$this->addControl($name, new CControl(array(
						"type" => $type
					)));
				}
			}
		}

		/**	Adds SUBMIT/CANCEL buttons.
		 * 	Besides, it adds all work stuff.
		 * 	@access	public
		 * 	@param	string	- name.
		 * 	@param	string	- name.
		 */
		function addSubmitCancel ($submit=SUBMIT, $cancel=CANCEL) {
			global $HTTP_REFERER;
			$this->addControl(-1, new CMarkup($this->o->startNoLabel($this,$dummy)));
			$this->addControl("submit", new CButton(array(
				"label"	=> $submit
			)));
			$this->addControl("cancel", new CButton(array(
				"label"	=> $cancel
			)));
			$this->addControl("referer", new CControl(array(
				"type"	=> "hidden",
				"default" => $HTTP_REFERER
			)));
			$this->addControl(-1, new CMarkup($this->o->endNoLabel($this,$dummy)));
		}

		/**	Sets the form mode.
		 * 	Besides, it adds all work stuff.
		 * 	@access	public
		 * 	@param	string	- new mode.
		 * 	@return	string	- old mode.
		 */
		function setMode ($newMode) {
			$this->mode = $newMode;
			$this->addControl("mode", new CControl(array(
				"label"	=> " ",
				"type"	=> "hidden",
				"default" => $newMode,
				"nodata" => TRUE
			)));
			global $mode;
			return $mode;
		}

		#//////	DATA HANDLING //////#

		/**	Returns value from the specified control.
		 * 	Use this to override the data handling,
		 * 	e.g. to process special data formats.
		 * 	@access	private
		 * 	@param	object	CControl
		 * 	@return	mixed	- value.
		 */
		function getValue ($control) {
			return $control->getValue();
		}

		/**	Control's value to input to form.
		 *	Use this to override the data handling,
		 *	e.g. to process special data formats.
		 * 	@access	private
		 *	@param	object 	CControl
		 *	@param	mixed	- value.
		 *	@return	mixed	- in value.
		 */
		function inValue ($control, $value) {
			return $control->inValue($value);
		}

		/**	Control's value to output from the form.
		 *	Use this to override the data handling,
		 *	e.g. to process special data formats.
		 * 	@access	private
		 *	@param	object 	CControl
		 *	@return	mixed	- out value.
		 */
		function outValue ($control) {
			return $control->outValue();
		}

		/**	Collects output values of the form.
		 *	Use this to get form data as array.
		 * 	@access	private
		 *	@return	array	- out values.
		 */
		function outValues () {
			$values = array();
			foreach ($this->controls as $control) {
				if (!$control->nodata) {
					$values[$control->name] = $this->outValue($control);
				}
			}
			return $values;
		}

		/**	Use this to validate form data.
		 * 	@access	public
		 * 	@return	bool	- TRUE if error occured.
		 */
		function validateValues () {
			$this->errors = array();
			foreach ($this->controls as $control) {
				if (!$control->nodata) {
					if ($errmsg = $this->validateValue($control)) {
						$this->errors[$control->name] = $errmsg;
					}
				}
			}
			return empty($this->errors);
		}

		/**	Use this to override the data verification on the form level,
		 *	e.g. to process special data formats.
		 * 	@access	private
		 *	@param	object 	CControl
		 *	@return	mixed	- control validation result.
		 */
		function validateValue ($control) {
			if (@$control->ValidateAsEqual
				&& $this->getValue($control) != $this->getValue($this->controls[$control->ValidationEqualSource])) {
				$msg = $control->ValidateAsEqual;
				eval ("\$msg = \"$msg\";");
				return $msg;
			}
			return $control->validateValue();
		}

		/*function write ($s) {echo "$s\n";} /*DEBUG*/

		#//////	RENDERING //////#

		/**	Main rendering method.
		 * 	Use this to render the whole form body.
		 * 	@access	public
		 */
		function render () {
			$this->renderStart();
			foreach ($this->controls as $control) {
				$this -> renderStartControl	($control);
				$this -> renderControl		($control);
				$this -> renderEndControl 	($control);
			}
			$this->renderEnd();
		}

		/**	Use this to insert the form caption manually
		 * 	and/or to override the method.
		 * 	@access	private
		 */
		function renderStart () {
			$this->write($this->o->start($this));
		}

		/**	Use this to insert the form ending manually
		 * 	and/or to override the method.
		 * 	@access	private
		 */
		function renderEnd () {
			$this->write($this->o->end($this));
		}

		/**	Use this to insert controls manually
		 *	<pre>$form->renderControl($form->controls["name"]);</pre>
		 *	and/or to override the method. It calls
		 *	<code>$control->{"render$this->mode"}()</code> by default.
		 * 	@access	public
		 */
		function renderControl (&$control) {
			$control->{"render$this->mode"}();
		}

		/**	Use this to insert the control prefix (e.g. label) manually
		 *  <pre>$form->renderStartControl($form->controls["name"]);</pre>
		 * 	and/or to override the method.
		 * 	@access	private
		 */
		function renderStartControl (&$control) {
			if (@$control->label && @$control->type != "hidden") {
				$this->write($this->o->startControl($this,$control));
			}
		}

		/**	Use this to insert the control ending manually
		 *	<pre>$form->renderEndControl($form->controls["name"]);</pre>
		 * 	and/or to override the method.
		 * 	@access	private
		 */
		function renderEndControl (&$control) {
			if (@$control->label && @$control->type != "hidden") {
				$this->write($this->o->endControl($this,$control));
			}
		}
	}
?>