<?php
	/*'************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	/**	Basic form handler.
	 * 	It renders the associated form as simple succession
	 * 	<xmp>
	 * 	<p>label</p>
	 * 	<p><input ...></p>
	 * 	</xmp>
	 * 	The presentation is controlled by CSS:
	 * 	<pre><ul>
	 * 	<li> .formrow	- the whole form row (label+field) class;
	 * 	<li> .required	- required field label class;
	 * 	<li> .label		- ordinary field label class;
	 * 	<li> .nolabel	- labeless field (usually button) class;
	 * 	<li> .field		- the ordinary field class;
	 * 	</pre></ul>
	 * 	@package	PHP/Toolbox::Form
	 */
	class CFormHandlerSimple {
		/**	Use this to insert the form caption manually
		 *  <xmp>$form->renderStart();</xmp>
		 * 	and/or to override the method.
		 */
		function start (&$form) {
			return ("<form action='$form->action' method='$form->method' $form->attrs>");
		}

		/**	Use this to insert the form ending manually
		 * 	and/or to override the method.
		 */
		function end (&$form) {
			return ("</form>");
		}

		/**	Use this to insert the control prefix (e.g. label) manually
		 * 	and/or to override the method.
		 *  <pre>$form->renderStartControl($form->controls["name"]);</pre>
		 */
		function startControl (&$form, &$control) {
			return ("<p class=formrow><div class="
				.(isset($control->ValidateAsNotEmpty) ? "required" : "label")
				.">$control->label</div><div class=field>");
		}

		/**	Use this to insert the control ending manually
		 * 	and/or to override the method.
		 *	<pre>$form->renderEndControl($form->controls["name"]);</pre>
		 */
		function endControl (&$form, &$control) {
			return ("</div></p>");
		}

		/**	Use this to insert the 'labeless' starting manually
		 * 	and/or to override the method.
		 */
		function startNoLabel (&$form, &$control) {
			return "<p class=formrow><div class=nolabel>";
		}

		/**	Use this to insert the 'labeless' ending manually
		 * 	and/or to override the method.
		 */
		function endNoLabel (&$form, &$control) {
			return "</div></p>";
		}
	}
?>