<?php
	/*'************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once	"cformhandlersimple.phps";

	/**	Reference Implementation Of The Tabbed Form Handler.
	 * 	It renders the associated form into table 'gizmo'.
	 * 	<xmp>
	 * 	<tr><td>label</td>
	 * 	<td><input ...></td><tr>
	 * 	</xmp>
	 * 	The presentation is controlled by CSS:
	 * 	<pre><ul>
	 * 	<li> .backtable	- background table class;
	 * 	<li> .backcell	- background table cell (TD) class;
	 * 	<li> .backform	- the form background class;
	 * 	<li> .formrow
	 * 	<li> .required
	 * 	<li> .label
	 * 	<li> .nolabel
	 * 	<li> .field
	 * 	</ul></pre>
	 * 	@package	PHP/Toolbox::Form
	 */
	class CFormHandlerTabbed extends CFormHandlerSimple {
		/**	Use this to insert the form caption manually
		 *  <xmp>$form->renderStart();</xmp>
		 * 	and/or to override the method.
		 */
		function start (&$form) {
			return CFormHandlerSimple::start($form)
				.("<table border=0 class=backtable cellspacing=0 cellpadding=1><tr><td class=backcell>")
				.("<table border=0 class=backform  cellspacing=0 cellpadding=4 width=100%>");
		}

		/**	Use this to insert the form ending manually
		 * 	and/or to override the method.
		 */
		function end (&$form) {
			return ("</table></td></tr></table>")
				.CFormHandlerSimple::end($form);
		}

		/**	Use this to insert the control prefix (e.g. label) manually
		 * 	and/or to override the method.
		 *  <pre>$form->renderStartControl($form->controls["name"]);</pre>
		 */
		function startControl (&$form, &$control) {
			return ("<tr class=formrow><td class="
				.(isset($control->ValidateAsNotEmpty) ? "required" : "label")
				.">$control->label</td><td class=field>");
		}

		/**	Use this to insert the control ending manually
		 * 	and/or to override the method.
		 *	<pre>$form->renderEndControl($form->controls["name"]);</pre>
		 */
		function endControl (&$form, &$control) {
			return (isset($form->errors[$control->name]) ? ("<br>".$form->errors[$control->name]) : "")."</td></tr>";
		}

		/**	Use this to insert the 'labeless' starting manually
		 * 	and/or to override the method.
		 */
		function startNoLabel (&$form, &$control) {
			return "<td class=nolabel colspan=2>";
		}

		/**	Use this to insert the 'labeless' ending manually
		 * 	and/or to override the method.
		 */
		function endNoLabel (&$form, &$control) {
			return "</td>";
		}
	}