<?php
	/*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	/**	The root html snippet class.
	 * 	It provides some core functionality -- in particular construction with
	 * 	list of arbitrary fields. Use its private methods to override the default
	 * 	behavior of the CSnippet-based classes.
	 * 	@package	PHP/Toolbox
	 */
	class CSnippet {
		/**	Id allows distinguish the CSnippet's instances on the page
		 * 	e.g. if an array of instances of the same form is needed, sort of.
		 */
		var $id = "";

		/**	Constructs the snippet
		 * 	from arbitrary field list, e.g
		 * 	<xmp>
		 |	$snippet = new CSnippet(array(
		 |		"custom_field_1" => "custom_value_1,
		 |		"custom_field_2" => "custom_value_2,
		 |		"custom_field_3" => "custom_value_3,
		 |		...
		 |	));
		 * 	</xmp>
		 * 	It just uses append() method.
		 * 	@param	array	- list (array) of fields.
		 * 	@access	public
		 * 	@see append()
		 */
		function CSnippet ($fields=array()) {
			if ($fields) $this->append($fields);
		}

		/**	Appends arbitrary field list to the snippet.
		 * 	@param	array	- list (array) of fields.
		 * 	@access	public
		 */
		function append ($fields) {
			foreach ($fields as $key => $value) {
				$this->$key = $value;
			}
		}

		/**	Qualifies input name by snippet id.
		 * 	@param	string	- the name to be qualified.
		 * 	@return	string	- $this->id.$varName.
		 */
		function qualify ($varName) {
			return $this->id.$varName;
		}

		/**	Returns the value from global namespace or default
		 * 	if global variable is not set.
		 * 	@param	string
		 * 	@param	mixed	- default value
		 * 	@return	mixed
		 */
		function getGlobal ($varName, $default = "") {
			//	Oops! $$name breaks global binding to $name...
			$_name_of_var = $this->qualify($varName);
			global $$_name_of_var;
			return isset($$_name_of_var) ? $$_name_of_var : $default;
		}

		/**	Sets the value to global namespace variable.
		 * 	@param	string
		 * 	@param	mixed	- new value.
		 * 	@return	mixed	- new value.
		 */
		function setGlobal ($varName, $value) {
			$_name_of_var = $this->qualify($varName);
			global $$_name_of_var;
			return $$_name_of_var = $value;
		}

		function writeCode ($code, $eval=FALSE) {
			$this->write($eval ? eval($code) : $code);
		}

		/**	Just 'echoes' the string by default.
		 * 	Use it to override this default behavior,
		 * 	e.g. to redirect output to file.
		 * 	@return	string	- input string.
		 */
		function write ($str) {
			echo $str;
			return $str;
		}

		/**	Just flushes the output buffer by default.
		 * 	Use it to override this default behavior,
		 * 	e.g. to flush to file.
		 */
		function flush () {
			flush();
		}
	}
?>