<?php
	/*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once "func.phps";
	require_once "csnippet.phps";

	/**	The all-purpose class for programmatic pagination.
	 * 	It uses template substitution technique to generate the text 'buttons':
	 * 	<xmp>
	 - 	<FIRST>...</FIRST> - goto first page.
	 - 	<PREV>...</PREV> - goto previous page.
	 - 	<NEXT>...</NEXT> - goto next page.
	 - 	<LAST>...</LAST> - goto last page.
	 - 	<PAGELIST> - page list (e.g. 1 2 3 4 ...).
	 - 	<PAGESIZE> - size of the page.
	 - 	<TOTPAGES> - total number of pages.
	 - 	<ITEM-RANGE> - range of items on current page (e.g. 26-50).
	 * 	</xmp>
	 * 	Note that .curpage CSS-class is used to highlight current page.
	 * 	@package	PHP/Toolbox
	 */
	class CSplitBar
	extends CSnippet
	{
		var $SPLITBAR	= "<PREV>&lt;&lt;</PREV><PAGELIST><NEXT>&gt;&gt;</NEXT> (<ITEM-RANGE>)";
		var $PAGESIZE	= 0;
		var	$TOTROWS	= 0;
		var $PGOFFS		= 0;
		var	$PAGECOUNT	= 0;

		/**	Constructs the splitbar
		 * 	as, for example:
		 * 	<code>$sbar = new CSplitBar("SB1", 250, array("PAGESIZE"=>25));</code>
		 */
		function CSplitBar($id,	$TOTROWS, $attrs = array()) {
			$this->id = $id;
			$this->TOTROWS	= $TOTROWS;
			$this->CSnippet($attrs);
		}

		/**	Override it to provide custom template substitution.
		 * 	@param	string	- template string.
		 * 	@return	string	- substitution result.
		 */
		function subst_globals ($str) {
			return $str;//preg_replace ($search, $replace, $str);
		}

		/**	Main rendering method.
		 * 	If page size isn't specified then it calculated as min(max(ceil($this->TOTROWS/10),25),50).
		 */
		function render () {
			if (!$this->PAGESIZE) $this->PAGESIZE = min(max(ceil($this->TOTROWS/10),25),50);
			$this->PAGESIZE = min($this->PAGESIZE, $this->TOTROWS);
			//	looks like php4's 'global' breaks
			//	variable if the name is the same!
			$PG = $this->qualify("pg");
			global $$PG;
			if (!$$PG) $$PG = 1;
			$end = $$PG * $this->PAGESIZE;
			$this->PGOFFS = max(1,($$PG-1)*$this->PAGESIZE+1);

			$html = "";

			if ($this->TOTROWS) {
				$this->PAGECOUNT = ceil($this->TOTROWS/$this->PAGESIZE);
				$html = $this->subst_globals($this->SPLITBAR);
				/*bypass if there's no more than one page*/
				if ($this->TOTROWS > $this->PAGESIZE) {

					if ($end > $this->TOTROWS)
						$end = $this->TOTROWS;

	    			/* << First/Prev */
					if ($$PG <= 1) {
		    			$html = ereg_replace("</?(PREV|FIRST)>", "", $html);
					}
	    			else {
		    			$prevpage=$$PG-1;
		    			$html = str_replace("<FIRST>",
							"<a href='$PHP_SELF?".query_string(array($PG=>1))."'>", $html);
		    			$html = str_replace("<PREV>",
							"<a href='$PHP_SELF?".query_string(array($PG=>$prevpage))."'>", $html);
		    			$html = ereg_replace("</(PREV|FIRST)>", "</a>", $html);
					}

					/* Pagelist */
					for ($i=0, $pagelist=""; $i<$this->TOTROWS; $i+=$this->PAGESIZE) {
	        			$pi=($i/$this->PAGESIZE)+1;

	  					if($pi==$$PG) {
	  						$pagelist .= "<font class=curpage>&nbsp;$pi&nbsp;</font>";
	  					}
						else {
							$pagelist .= "<a href='$PHP_SELF?"
								.query_string(array($PG=>$pi))
								."'>&nbsp;$pi&nbsp;</a>";
						}
					}
					$html = str_replace("<PAGELIST>", $pagelist, $html);

					/* Next/Last >> */
    				if (($this->TOTROWS - $$PG * $this->PAGESIZE) <= 0) {
		    			$html = ereg_replace("</?(NEXT|LAST)>", "", $html);
					}
    				else {
        				$nextpage=$$PG+1;
        				$html = str_replace("<NEXT>",
							"<a href='$PHP_SELF?".query_string(array($PG=>$nextpage))."'>", $html);
		    			$html = str_replace("<LAST>",
							"<a href='$PHP_SELF?".query_string(array($PG=>$this->PAGECOUNT))."'>", $html);
		    			$html = ereg_replace("</(NEXT|LAST)>", "</a>", $html);
					}
					$html = str_replace("<PAGE>", $$PG, $html);
					$html = str_replace("<TOTPAGES>", $this->PAGECOUNT, $html);
				}

   				$html = str_replace("<ITEM-RANGE>", "$this->PGOFFS-$end", $html);
			}

			return $html;
		}/*render*/
	}/*CSplitBar*/
?>