<?php
	/*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	require_once	"csnippet.phps";

	/**	The text menu class.
	 * 	It renders horizontal menu of <em>text</em> items.
	 * 	So, such a menu system is easy (and fast) to design, construct and load.
	 *	<xmp>
	 |	$menu = new CTextMenu(array(
	 |		"<a href='?ITEM=0'>item_1</a>",
	 |		"<a href='?ITEM=1'>item_2</a>",
	 |		...
	 |		), "menu", 620, 18, array("0","FFFFFF"), 10
	 |	);
	 |	...
	 |	$menu->render(20, 20);
	 *	</xmp>
	 * 	It uses div (IE) or layer (NN) tags to implement the items logic.
	 * 	@package	PHP/Toolbox
	 * 	@see	$ITEM
	 */
	class CTextMenu extends CSnippet {
		var	$items = array();
		/**	Use it to specify the name of global variable that tracks current item. */
		var $ITEM = "ITEM";
		var $agent;

		/**	Constructs the menu.
		 * 	@param	array	- item array: array("item_html_1", ...) is accepted by default.
		 * 	@param	string	- CSS class for menu items.
		 * 	@param	int		- total width of menu bar.
		 * 	@param	int		- height of menu bar.
		 * 	@param	array	- item colors: array(NORMAL,HIGHLIGHTED); array("0","FFFFFF") by default.
		 * 	@param	int		- the gap between items.
		 */
		function CTextMenu ($items, $menu_class, $width, $height, $bgcolor=array(), $gap=0) {
			$this->items = $items;
			$this->step = round($width/count($items));
			$this->width = $this->step - $gap;
			$this->height = $height;
			$this->gap = $gap;
			$this->bgcolor = $bgcolor ? $bgcolor : array("0","FFFFFF");
			$this->menu_class = $menu_class;
			$this->agent = $this->getAgent();
		}

		/**	Use it to override default (not very smart :) user agent detection.
		 * 	Main rendering mehtod use "render_item_$this->agent" method to render the items.
		 * 	So, it's possible to provide additional item rendering methods to utilize
		 * 	such a custom detection.
		 * 	@return	string
		 * 	@see	render(), render_item_msie()
		 */
		function getAgent () {
			global $HTTP_USER_AGENT;
			eregi("MSIE", $HTTP_USER_AGENT, $agent);
			return strtolower($agent[0]);
		}

		/**	Renders the menu.
		 * 	@param	int	- the menu <em>absolute</em> x-position.
		 * 	@param	int	- the menu <em>absolute</em> y-position.
		 * 	@access	public
		 */
		function render($left=0, $top=0) {
			$this->left = $left;
			$this->top = $top;
			$this->current = $this->getGlobal($this->ITEM);
			$render_func = "render_item_$this->agent";
			$i = 0;
			foreach ( $this->items as $key => $value ) {
				$this->$render_func($i++, $key, $value);
			}
		}

		/**	Returns the item.
		 * 	Use it to override default behavior.
		 * 	@param	int	- item number.
		 * 	@param	mixed	- item key (if it's specified as key=>value).
		 * 	@param	string	- item text.
		 * 	@return	string	- item html.
		 */
		function render_item($i, $key, $value) {
			return is_array($value) ? join("\n", $value) : $value;
		}

		/**	Renders the item for MSIE.
		 * 	@see	getAgent()
		 */
		function render_item_msie($i, $key, $value) {
			$this->write (
				"<div class='$this->menu_class' style='position: absolute;"
				." left: ".($this->left + $i * $this->step)."; top: $this->top;"
				." width: $this->width; height: $this->height;"
				." background-color: #"
				.($this->current == $i ?
					$this->bgcolor[1] :
					($this->bgcolor[0]
					.";' onmouseover='this.style.backgroundColor=0x".$this->bgcolor[1]
					."' onmouseout='this.style.backgroundColor=0x".$this->bgcolor[0]))
				."'>"
				.$this->render_item($i, $key, $value)
				."</div>\n"
			);
		}

		/**	Renders the item for NN.
		 * 	@see	getAgent()
		 */
		function render_item_($i, $key, $value) {
			$this->write (
				"<layer class='$this->menu_class'"
				." left=".($this->left + $i * $this->step)." top=$this->top"
				." width=$this->width height=$this->height"
				." bgcolor='#"
				.($this->current == $i ?
					$this->bgcolor[1] :
					($this->bgcolor[0]
					."' onMouseOver='this.bgColor=0x".$this->bgcolor[1]
					."' onMouseOut='this.bgColor=0x".$this->bgcolor[0]))
				."'>"
				.$this->render_item($i, $key, $value)
				."</layer>\n"
			);
		}
	}
?>