<?php
	/**
	 * @package	PHP/Toolbox
	 * @module	func.phps
	 *//*'*************************************************\
	 	Copyright (C) 2001, Alexey Ilyin

	 	This library is free software; you can
		redistribute it and/or modify it under the terms
		of the GNU Lesser General Public License as
		published by the Free Software Foundation; either
		version 2.1 of the License, or (at your option)
		any later version.

	 	This library is distributed in the hope that it
		will be useful, but WITHOUT ANY WARRANTY; without
		even the implied warranty of MERCHANTABILITY or
		FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
		Lesser General Public License for more details.

	 	You should have received a copy of the GNU Lesser
		General Public License along with this library; if
		not, write to the Free Software Foundation, Inc.,
		59 Temple Place, Suite 330, Boston, MA 02111-1307
		USA

	 	ai@a4web.com
	 	www.a4web.com
	\**************************************************/

	if (!defined("PHPXFUNC")):
	define("PHPXFUNC", "4.0");

    //  define useful constants
	//define ("NULL", "");
    define ("CR",	"\r");
    define ("LF",  	"\n");
    define ("CRLF",	"\r\n");
	define ("K", 	1024);
	define ("M", 	1048576);

	#=====	INFORMATION & DEBUGGING  =======================================#

	/**	Logs specified message to PROJECT_LOG_FILE if defined.
	 */
	function log_message ($message) {
		//	define("PROJECT_LOG_FILE", "/tmp/phpx.log");
		if (defined("PROJECT_LOG_FILE")) {
			error_log(date("Y-F-d h:i:s A")."\t$message\n", 3, PROJECT_LOG_FILE);
		}
	}

	/**	PHP/Toolbox custom assertion check routine.
	 * 	If condition is false then:
	 * 	<nl>
	 * 	<li>	logs the message;
	 * 	<li>	if show is true or SHOW_ASSERT_MESSAGES is defined then
	 * 			echoes the message with span class assert;
	 *			otherwise, if FAILED_ASSERTION_URL is defined then redirects to it;
	 * 	<li>	exits.
	 * 	</nl>
	 * 	@see	log_message()
	 */
	function phpx_assert ($condition, $message="ASSERTION FAILED", $show=FALSE) {
		if (!$condition) {
			log_message($message);
			if ($show || defined("SHOW_ASSERT_MESSAGES")) {
				echo "<span class=assert>$message</span>";
			}
			elseif (defined("FAILED_ASSERTION_URL")) {
				global $PHP_SELF;
    			@header("HTTP/1.0 302");
    			@header("Location: ".FAILED_ASSERTION_URL."?fn=$PHP_SELF"
					//."&msg=".htmlentities(urlencode($message))
				);
			}
			exit;
		}
	}

	/**	Logs the message and dies with it
	 * 	if show is true.
	 */
	function phpx_abort ($message="", $show=FALSE) {
		log_message($message);
		die($show ? "$message" : "");
	}

	/**	Prints the variable data.
	 * 	If show is false, it prints into html-comment.
	 */
	function phpx_debug ($v="DEBUG", $show=TRUE) {
		echo "\n", $show ? "<xmp>" : "<!--\n";
		if (is_string($v) || is_numeric($v)) echo $v;
		else print_r($v);
		echo $show ? "</xmp>" : "\n-->", "\n";
	}

	#=====	MISC ROUTINES  =======================================#

	/**	Calculates scaled dimensions for the specified image.
	 * 	@param	string
	 * 	@param	int		- max image width.
	 * 	@param	int		- max image height.
	 * 	@return	string	- "width=$scaled_width height=$scaled_height".
	 */
	function ImgScale ($img_file, $MAX_W = 150, $MAX_H = 150)
	{	//	needs local path
		list($width, $height, $img_info, $fullsize) = @getimagesize($img_file);
		if ($width > $MAX_W || $height > $MAX_H) {
			$ratio = $width / $height;
			if ($width > $MAX_W) {
	   			$width = $MAX_W;
				$height = $MAX_W / $ratio;
			}
			if ($height > $MAX_H) {
	   			$width = $MAX_H * $ratio;
				$height = $MAX_H;
			}
		}
		return "width=$width height=$height";
	}

	/**	Creates text 'button' to open child window
	 * 	regardless to javascript availability.
	 * 	@param	string	- link text.
	 * 	@param	string	- source document the link points to.
	 *	@param	string	- child window name.
	 * 	@param	string	- script window.open params.
	 * 	@return	string	- html.
	 */
	function SmartViewer ($text, $src, $wname="", $wparm="width=200, height=100")
	{
		return "
<a href='$src' target='$wname'
onclick='javascript:window.open(\"$src\",\"$wname\",\"$wparm\"); return false;'
>$text</a>";
	}

	/**	Checks if specified variable is set.
	 * 	<ul>
	 * 	<li>	If empty, returns "".
	 * 	<li>	Otherwise, if variable is a scalar, returns attr
	 * 			when *any* value is assigned or matched to specified one.
	 * 	<li>	Otherwise, (variable *is* an array) and key isn't specified,
	 * 			returns attr when the value is in array.
	 * 	<li>	Otherwise, (the key *is* specified),
	 * 			checks if *any* value is assigned or matched to specified one.
	 * 	</ul>
	 * 	@access	private
	 */
	function input_check ($name, $attr, $value="", $key="") {
		global $$name;
		if (empty($$name))
			return "";
		elseif (!is_array($$name))
			//	check if any value is assigned or matched to specified one
			return $value === "" || $$name == $value ? $attr : "";
		elseif ($key === "")
			//	check if value is matched to specified one
			return in_array($value, $$name) ? $attr : "";
		else
			//	check if any value is assigned or matched to specified one
			return ${$name}[$key] && ($value === "" || ${$name}[$key] == $value) ? $attr : "";
	}

	/**	Check if option selected.
	 * 	@return	string	- "selected" if there is variable matched.
	 * 	@see	input_check()
	 */
	function selected ($name, $value="", $key="") {
		return input_check ($name, "selected", $value, $key);
	}

	/**	Check if box checked.
	 * 	@return	string	- "checked" if there is variable matched.
	 * 	@see	input_check()
	 */
	function checked ($name, $value="", $key="") {
		return input_check ($name, "checked", $value, $key);
	}

	#=====	ELEMENT MANIPULATION  =======================================#

	/**	Sets the value to array item or object field if they are not set yet.
	 * 	@return	bool	- TRUE if value is really set.
	 */
	function set_if_empty (&$array, $key, $arg) {
		if (is_array($array)) {
			if (!isset($array[$key])) {
				$array[$key] = $arg;
				return TRUE;
			}
		}
		elseif (is_object($array)) {
			if (!isset($array->$key)) {
				$array->$key = $arg;
				return TRUE;
			}
		}
		return FALSE;
	}

	/**	Returns value or default.
	 * 	@returns	mixed	value or default
	 */
	function defValue (&$value, $def) {
		return isset($value) ? $value : $def;
	}

	/**	Concatenates *non-empty* args with infix.
	 * 	<code>concat_nonempty(array("a"=>1,"e"=>"","x"=>"eks","z"=>0))</code>
	 * 	yields "1,eks"
	 */
	function concat_nonempty ($arg, $infix=",")
	{
		$result = $inf = "";
		while (list($key,$val)=each($arg)) {
			if ($val) {
				$result .= "$inf$val";
				$inf = $infix;
			}
		}
		return $result;
	}

	/**	Concatenates *non-empty* args with infix
	 * 	for use in SQL-statements.
	 * 	<code>concat_nonempty_sql(array("a"=>1,"e"=>"","x"=>array("eks","cross"),"z"=>0))</code>
	 * 	yields "a='1',e=null,x='eks,cross',z='0'"
	 */
	function concat_nonempty_sql ($arg, $infix=",")
	{
		$result = $inf = "";
		while (list($key,$val)=each($arg)) {
			if (is_array($val)) {
				$val = join(",", $val);
			}
			$result .= "$inf$key=".($val === "" ? "'$val'" : "null");
			$inf = $infix;
		}
		return $result;
	}

	/**	substitutes subs/str or key=>value pairs in string
	 *	and returns result:
	 *	<li> <code>substitute (array("a"=>1,"b"=>2), "x:a; y:b")</code> yields "x:1; y:2"
	 *	<li> <code>substitute ("1", "a", "x:a")</code> yields "x:1"
	 *  <p>
	 *	 note that this function doesn't use pcre
	 *	 but is slower than preg_replace.
	 * 	@param	array	- key=>value pairs or just string to substitute.
	 * 	@param	string	- source string or substring to be substituted.
	 * 	@return	string
	 */
	function substitute ($subs, $str, $str2="") {
		if (is_string($subs)) {
			$result = ereg_replace($subs, $str, $str2);
		}
		else {
			$subs = (array)$subs;
			$result = $str;
			while (list($key,$val)=each($subs)) {
				$result = ereg_replace($key, $val, $result);
			}
		}
		return $result;
	}

	/**	Adds/replaces key=>value pairs to/in http-query strings.
	 * 	<ul>
	 * 	<li>
	 * 	<code>phpx_urlencode("name=John Smyth&age=32")</code>
	 * 			yields "name=John+Smyth&age=32"
	 * 	<li>
	 * 	<code>phpx_urlencode(array("name"=>"John Smyth","age"=>32),array("age"=>33))</code>
	 * 			yields "name=John+Smyth&age=33"
	 * 	</ul>
	 * 	Use
	 * 	<xmp>
	 * 	echo "?", phpx_urlencode($HTTP_GET_VARS, array("age"=>33))
	 * 	</xmp>
	 * 	to correctly set 'get' parameter(s).
	 * 	@see	query_string()
	 */
	function phpx_urlencode ($arg, $replacements=0) {
		if (is_string($arg)) {
			return urlencode($arg);
		}
		if (!is_array($replacements)) {
			$arg = array_merge($arg, $replacements);
		}
		$result = $amp = "";
		foreach ($arg as $name => $value) {
			$result .= "$amp$name=".urlencode($value);
			$amp = "&";
		}
		return $result;
	}

	/**	Adds/replaces key=>value pairs in $QUERY_STRING
	 *	and returns result:
	 *	<li> <code>query_string (array("a"=>1,"b"=>2))</code> yields "a=1&b=2"
	 *	<li> <code>query_string (array("a"=>1,"b"=>2), "a=x")</code> yields "a=1&b=2"
	 * 	</li>
	 * 	@param	array	- arguments.
	 * 	@param	string	- it's used instead of default $QUERY_STRING global.
	 * 	@return	string
	 * 	@see phpx_urlencode()
	 */
	function query_string ($args, $qstr=0) {
		if (is_integer($qstr)) {
			global $QUERY_STRING;
			$qstr = $QUERY_STRING;
		}
		$result = $amp = "";
		while (list($name,$value) = each($args)) {
			if (empty($value)) continue;
			$result .= "$amp$name=$value";
			$amp = "&";
		}
		if ($qstr) {
			$vars = split("&|=", $qstr);
			while (list($i,$name) = each($vars)) {
				list($i,$value) = each($vars);
				if (isset($args[$name])) continue;
				if (empty($value)) continue;
				$result .= "$amp$name=$value";
				$amp = "&";
			}
		}
		return $result;
	}

	/**	 collects form field values from
	 	<li> global space (e.g. <code>collect_fields(array("name"=>"term(name)"),"infix")</code>) or
	 	<li> specified array/object (e.g. <code>collect_fields(array("name"=>"term(name)"),$record,"infix")</code>)
	 *  </li>
		and returns result:
	 	<xmp> collect_fields(array("a"=>"a='\$a'","b"=>"abs('\$b')),[$rec,]"&&") yields "a=1&&abs(2)" </xmp>
	 * 	@param	array	- field descriptions, e.g. <code>array("a"=>"a='\$a'", "b"=>"abs('\$b'))</code>.
	 * 					Note that they are eval'ed.
	 * 	@param	array	- fields are collected from it if it's specified.
	 * 	@param	string	- infix.
	 * 	@return	string
	 */
	function collect_fields ($field_descr, $arg=" ", $arg2=" ") {
		$result = $infix = "";
		if (is_string($arg)) {
			//  collect from global space
			while (list($field, $term) = each($field_descr)) {
				global $$field;
				if (isset($$field) && $term) {
					eval("\$result .= \"$infix$term\";");
					$infix = $arg;
				}
			}
		}
		else {
			//	collect from specified array/object
			$arg = (array)$arg;
			while (list($field, $term) = each($field_descr)) {
				if (isset($arg[$field]) && $term) {
					eval("\$result .= \"$infix$term\";");
					$infix = $arg2;
				}
			}
		}
		return $result;
	}

	/**	Collects fields to use in SQL-statement 'set' clause
	 * 	from the global space or from the array if specified.
	 * 	@param	array	- field descriptions as
	 * 					<code>array(name[=>operator],...)</code>.
	 * 					Note, operator is eval'ed, e.g.
	 * 					<code>"abs(\$value)"</code> leads the SQL apply
	 * 					'abs' function to the value passed.
	 * 	@return	string	- e.g. "a='1',b=abs('-2'),c='1'+'-2'"
	 */
	function collect_fields_sql ($fields, $array=array()) {
		$result = $infix = "";
		foreach ($fields as $name => $operator) {
			if (is_int($name)) {
				$name = $operator;
				$operator = "'\$value'";
			}
			if ($array) {
				$value = $array[$name];
			}
			else {
				global $$name;
				$value = $$name;
			}
			if (is_array($value)) {
				$value = join(",", $value);
			}
			eval("\$result .= \"$infix$name=$operator\";");
			$infix = ",";
		}
		return $result;
	}

	/**	Extracts values from the array into global namespace.
	 * 	Nested arrays are converted into CSV-strings.
	 */
	function extract_fields_sql (&$record) {
		$result = $infix = "";
		foreach ($record as $name => $value) {
			global $$name;
			if (is_array($value)) {
				$value = join(",", $value);
			}
			$$name = $value;
		}
	}

	/**	returns 18-xdigit fair unique string
	 * 	@return	string
	 */
	function UniqIdent ()
	{
		return uniqid(mt_rand(0x7000,0xFFFF));
	}

	#=====	NETWORK ROUTINES  =======================================#

	/**	Forces basic authentication procedure.
	 * 	@param	string	- the realm as RFCxxx requires.
	 */
	function WWW_Authenticate ($realm)
	{
		global $PHP_AUTH_USER;
		if (empty($PHP_AUTH_USER)) {
    		Header("WWW-Authenticate: Basic realm=\"$realm\"");
    		Header("HTTP/1.0 401 ".UNAUTHORIZED);
			die(
				"<html>\n<body>\n<h1>$realm</h1>\n"
				.WWW_AUTHENTICATION_REQUIRED.".\n</body>\n</html>"
			);
		}
	}

	#=====	DATABASE ROUTINES =======================================#

	require_once (@$SQL_SERVER ? $SQL_SERVER : "mysql").".phps";

	endif;//PHPXFUNC
?>